package com.idea.buildManage.controller;

import com.github.pagehelper.PageInfo;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.buildManage.service.ParkRoomService;
import com.rockstar.common.base.BaseController;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;

/**
 * Created by pengyq on 2020.
 */
@Controller
@RequestMapping(value = "ParkRoomController")
@Api(value="房间管理")
public class ParkRoomController extends BaseController {

    //主表
    @Autowired
    private ParkRoomService modelService;


    @PostMapping(value = "list",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object list(Tablepar tablepar, ParkRoom model, DateTrans dt){
        PageInfo<ParkRoom> page= modelService.list(tablepar,model,dt);
        TableSplitResult<ParkRoom> result = new TableSplitResult<ParkRoom>(page.getPageNum(), page.getTotal(), page.getList());
        return  result;
    }

    @ApiOperation(value = "房间列表")
    @PostMapping(value = "listAll",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object listAll(ParkRoom model, DateTrans dt){
        return  modelService.listAll(model,dt);
    }

    @ApiOperation(value = "新增房间")
    @PostMapping(value = "add",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult add(ParkRoom record){
        record.setCreatedAt(new Date());
        record.setCreatedId(ShiroUtils.getUserId());
        // 默认待售
        record.setSoldStatus(1);
        int result = modelService.insert(record);
        return result(result);
    }

    @ApiOperation(value = "删除房间")
    @PostMapping(value = "remove/{id}",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult remove(@PathVariable("id") String id){
        int result = modelService.deleteByPrimaryKey(id);
        return result(result);
    }

    @ApiOperation(value = "编辑房间")
    @PostMapping(value = "edit",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult editSave(ParkRoom model){
        model.setUpdatedAt(new Date());
        model.setUpdatedId(ShiroUtils.getUserId());
        int result = modelService.edit(model);
        return result(result);
    }

    @ApiOperation(value = "查询房间")
    @PostMapping(value = "getById",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public ParkRoom getById(String id) {
        return modelService.selectByPrimaryKey(id);
    }

    @ApiOperation(value = "房间列表分页")
    @PostMapping(value = "listByModel",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object listByModel(Tablepar tablepar, ParkRoomResponse model, DateTrans dt){
        PageInfo<ParkRoomResponse> page= modelService.listByModel(tablepar,model,dt);
        TableSplitResult<ParkRoomResponse> result = new TableSplitResult<ParkRoomResponse>(page.getPageNum(), page.getTotal(), page.getList());
        return  result;
    }

    @ApiOperation(value = "全房间列表")
    @PostMapping(value = "excelList",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object excelList(ParkRoomResponse model){
        return success(modelService.excelList(model,null));
    }


    /**
     * 获取房屋面积
     * @param ids
     * @return
     */
    @PostMapping(value = "getAreaByIds",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Double getAreaByIds(String ids){

        return modelService.getAreaByIds(ids);
    }


}
