package com.idea.buildManage.excel;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSONObject;
import com.idea.buildManage.model.MnpBuilding;
import com.idea.buildManage.service.MnpBuildingService;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkInfoService;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class MnpBuildingListener extends AnalysisEventListener<MnpBuildingExcel> {

    private List<MnpBuilding> list = new ArrayList<>();

    private List<MnpBuildingExcel> excelList = new ArrayList<>();

    private List<MnpBuildingExcel> badList = new ArrayList<>();

    Integer count;

    private JSONObject jsonObject;

    private MnpBuildingService buildingService;
    private ParkFloorDiscService floorDiscService;
    private ParkInfoService parkInfoService;
    private SysDictService sysDictService;


    public MnpBuildingListener(MnpBuildingService buildingService,ParkFloorDiscService floorDiscService,
                               ParkInfoService parkInfoService,SysDictService sysDictService, JSONObject jsonObject) {
        this.buildingService = buildingService;
        this.floorDiscService = floorDiscService;
        this.parkInfoService = parkInfoService;
        this.sysDictService = sysDictService;
        this.jsonObject = jsonObject;
    }


    @Override
    public void invoke(MnpBuildingExcel mnpBuildingExcel, AnalysisContext analysisContext) {
        excelList.add(mnpBuildingExcel);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {


        // 入驻类型字典
        List<Map<String, String>> buildStructure = sysDictService.selectDictList("BUILD_STRUCTURE");
        if(CollectionUtils.isNotEmpty(excelList)){
            MnpBuilding data;
            MnpBuildingExcel excel = null;
            Map<String, String> discNameIdMap = floorDiscService.getNameIdMap();
            Map<String, String> parkNameIdMap = parkInfoService.getNameIdMap();
            for(int i = 0; i < excelList.size(); i++){
                try {
                    data = new MnpBuilding();
                    excel = excelList.get(i);
                    if (StringUtils.isEmpty(excel.getGroupName())) {
                        throw new RuntimeException("楼盘/小区名称为空");
                    }
                    data.setGroupName(excel.getGroupName());
                    if (StringUtils.isEmpty(excel.getDiscName())) {
                        throw new RuntimeException("分期名称为空");
                    }
                    data.setDiscName(excel.getDiscName());
                    if (StringUtils.isEmpty(excel.getBuildNum())) {
                        throw new RuntimeException("单元楼栋号为空");
                    }
                    data.setBuildNum(excel.getBuildNum());
                    // 判断小区是否存在
                    if(StringUtils.isEmpty(parkNameIdMap.get(excel.getGroupName()))){
                        throw new RuntimeException("楼盘/小区不存在");
                    }
                    data.setGroupId(parkNameIdMap.get(excel.getGroupName()));
                    // 判断分期是否存在
                    if(StringUtils.isEmpty(discNameIdMap.get(excel.getGroupName()))){
                        throw new RuntimeException("分期不存在");
                    }
                    data.setDiscId(discNameIdMap.get(excel.getGroupName()));
                    // 总面积
                    if (StringUtils.isEmpty(excel.getArea())) {
                        throw new RuntimeException("总面积为空");
                    }
                    try {
                        data.setArea(Double.valueOf(excel.getArea()));
                    }catch (NumberFormatException e){
                        throw new RuntimeException("总面积格式有误");
                    }
                    // 住宅面积
                    if (StringUtils.isEmpty(excel.getResidentialArea())) {
                        throw new RuntimeException("住宅面积为空");
                    }
                    try {
                        data.setResidentialArea(Double.valueOf(excel.getResidentialArea()));
                    }catch (NumberFormatException e){
                        throw new RuntimeException("住宅面积格式有误");
                    }
                    // 非住宅面积
                    if (StringUtils.isEmpty(excel.getUnResidentialArea())) {
                        throw new RuntimeException("非住宅面积为空");
                    }
                    try {
                        data.setUnResidentialArea(Double.valueOf(excel.getUnResidentialArea()));
                    }catch (NumberFormatException e){
                        throw new RuntimeException("非住宅面积格式有误");
                    }
                    // 占地面积
                    if (StringUtils.isEmpty(excel.getCoverAnArea())) {
                        throw new RuntimeException("占地面积为空");
                    }
                    try {
                        data.setCoverAnArea(Double.valueOf(excel.getCoverAnArea()));
                    }catch (NumberFormatException e){
                        throw new RuntimeException("占地面积格式有误");
                    }
                    // 地下面积
                    if (StringUtils.isEmpty(excel.getUndergroundArea())) {
                        throw new RuntimeException("地下面积为空");
                    }
                    try {
                        data.setUndergroundArea(Double.valueOf(excel.getUndergroundArea()));
                    }catch (NumberFormatException e){
                        throw new RuntimeException("地下面积格式有误");
                    }
                    // 房号
                    if (StringUtils.isNotEmpty(excel.getRoomNum())) {
                        data.setRoomNum(excel.getRoomNum());
                    }
                    // 房号
                    if (StringUtils.isNotEmpty(excel.getRoomNum())) {
                        throw new RuntimeException("房号为空");
                    }
                    data.setRoomNum(excel.getRoomNum());
                    // 套数
                    if (StringUtils.isNotEmpty(excel.getNumberInfo())) {
                        throw new RuntimeException("套数为空");
                    }
                    data.setNumberInfo(excel.getNumberInfo());
                    // 总层数
                    if (StringUtils.isNotEmpty(excel.getFloors())) {
                        throw new RuntimeException("总层数为空");
                    }
                    try {
                        data.setFloors(Integer.valueOf(excel.getFloors()));
                    }catch (NumberFormatException e){
                        throw new RuntimeException("总层数格式有误");
                    }
                    // 地上层数
                    if (StringUtils.isNotEmpty(excel.getOvergroundFloors())) {
                        throw new RuntimeException("地上层数为空");
                    }
                    try {
                        data.setOvergroundFloors(Integer.valueOf(excel.getOvergroundFloors()));
                    }catch (NumberFormatException e){
                        throw new RuntimeException("地上层数格式有误");
                    }
                    // 地下层数
                    if (StringUtils.isNotEmpty(excel.getUndergroundFloors())) {
                        throw new RuntimeException("地下层数为空");
                    }
                    try {
                        data.setUndergroundFloors(Integer.valueOf(excel.getUndergroundFloors()));
                    }catch (NumberFormatException e){
                        throw new RuntimeException("地下层数格式有误");
                    }
                    // 结构
                    if (StringUtils.isNotEmpty(excel.getStructure())) {
                        throw new RuntimeException("结构为空");
                    }
                    for (Map<String, String> stringStringMap : buildStructure) {
                        if (stringStringMap.get("label").equals(excel.getStructure())) {
                            data.setStructure(stringStringMap.get("value"));
                        }
                    }
                    // 没匹配上的情况
                    if(StringUtils.isEmpty(data.getStructure())){
                        throw new RuntimeException("第" + (i + 3) + "行，入驻类型格式有误");
                    }
                    // 行政区划
                    if (StringUtils.isNotEmpty(excel.getRegionalismCode())) {
                        throw new RuntimeException("行政区划为空");
                    }
                    data.setRegionalismCode(excel.getRegionalismCode());
                    // 地号
                    if (StringUtils.isNotEmpty(excel.getLandCode())) {
                        throw new RuntimeException("地号为空");
                    }
                    data.setLandCode(excel.getLandCode());
                    data.setRemark(excel.getRemark());
                    data.setPriceRemark(excel.getPriceRemark());
                    count = list.size();
                    jsonObject.put("successCount", count);
                    jsonObject.put("errorList", badList);
                    jsonObject.put("errorCount", badList.size());
                    //保存
                    saveData();

                    excelList.clear();
                    list.clear();

                }catch (Exception e){
                    e.printStackTrace();
                    String message = e.getMessage();
                    excel.setReason(message);
                    badList.add(excel);
                }
            }
        }

    }

    private void saveData(){
        if (CollectionUtils.isNotEmpty(list)) {
            buildingService.insertBatch(list);
        }
    }


}
