package com.idea.buildManage.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.response.ParkFloorDiscRespson;
import com.idea.buildManage.mapper.ParkFloorDiscExtendMapper;
import com.idea.buildManage.mapper.ParkFloorDiscMapper;
import com.idea.buildManage.model.*;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.SnowflakeIdWorker;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ParkFloorDiscService implements BaseService<ParkFloorDisc, ParkFloorDiscExample> {

    //文件mapper
    @Autowired
    private ParkFloorDiscMapper modelMapper;
    @Autowired
    private MnpBuildingService buildingService;
    @Autowired
    private ParkFloorDiscExtendMapper extendMapper;
    @Autowired
    private HouseTypeService houseTypeService;

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<ParkFloorDisc> list(Tablepar tablepar, ParkFloorDisc model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ParkFloorDisc> list = modelMapper.selectByExample(getCondition(model, dt));
        PageInfo<ParkFloorDisc> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询指定
     *
     * @return
     */
    public List<ParkFloorDisc> listAll(ParkFloorDisc model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    private ParkFloorDiscExample getCondition(ParkFloorDisc model, DateTrans dt) {
        ParkFloorDiscExample ex = new ParkFloorDiscExample();
        ex.setOrderByClause("created_at asc");
        ParkFloorDiscExample.Criteria criteria = ex.createCriteria();

        return ex;
    }

    @Override
    public int deleteByPrimaryKey(String id) {

        // 先查询楼盘下是否有楼栋
        MnpBuildingExample buildingExample = new MnpBuildingExample();
        buildingExample.createCriteria().andDiscIdEqualTo(id);
        long count = buildingService.countByExample(buildingExample);
        if (count > 0) {
            throw new RuntimeException("该楼盘下存在楼栋,无法删除!");
        }
        return modelMapper.deleteByPrimaryKey(id);
    }

    @Override
    public ParkFloorDisc selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }

    @Override
    public int updateByPrimaryKeySelective(ParkFloorDisc record) {
        record.setUpdatedAt(new Date());
        // 关联载体修改楼盘名称
        MnpBuildingExample mnpBuildingExample = new MnpBuildingExample();
        mnpBuildingExample.createCriteria().andDiscIdEqualTo(record.getId());
        List<MnpBuilding> mnpBuildings = buildingService.selectByExample(mnpBuildingExample);
        for(MnpBuilding build: mnpBuildings){
            build.setDiscName(record.getName());
            buildingService.updateByPrimaryKey(build);
        }
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(ParkFloorDisc record, ParkFloorDiscExample example) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(ParkFloorDisc record, ParkFloorDiscExample example) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<ParkFloorDisc> selectByExample(ParkFloorDiscExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(ParkFloorDiscExample example) {
        return modelMapper.countByExample(example);
    }

    public long countByExample(ParkFloorDisc model, DateTrans dt) {
        return modelMapper.countByExample(getCondition(model, dt));
    }

    @Override
    public int deleteByExample(ParkFloorDiscExample example) {
        return modelMapper.deleteByExample(example);
    }

    public int insert(ParkFloorDisc record) {
        record.setId(SnowflakeIdWorker.getUUID());
        return insertWithoutId(record);
    }

    public int insertWithoutId(ParkFloorDisc record) {
        record.setCreatedAt(new Date());
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(ParkFloorDisc record) {
        record.setId(SnowflakeIdWorker.getUUID());
        record.setCreatedAt(new Date());
        return modelMapper.insertSelective(record);
    }

    public PageInfo<ParkFloorDiscRespson> listByModel(Tablepar tablepar, ParkFloorDiscRespson model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ParkFloorDiscRespson> list = extendMapper.listByModel(model);
        PageInfo<ParkFloorDiscRespson> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 新增分期
     * @param model
     * @return
     */
    public int add(ParkFloorDiscRespson model){
        String houseTypeStr = model.getHouseTypeStr();
        String id = IdUtil.simpleUUID();
        model.setId(id);
        List<HouseType> houseTypes = JSONObject.parseArray(houseTypeStr, HouseType.class);
        for(HouseType houseType : houseTypes){
            houseType.setId(IdUtil.simpleUUID());
            houseType.setDiscId(id);
            houseTypeService.insertSelective(houseType);
        }
        model.setCreatedId(ShiroUtils.getUserId());
        return insertWithoutId(model);
    }

    /**
     * 编辑分期
     * @param model
     * @return
     */
    public int edit(ParkFloorDiscRespson model){

        //先删除对应户型库
        HouseTypeExample houseTypeExample = new HouseTypeExample();
        houseTypeExample.createCriteria().andDiscIdEqualTo(model.getId());
        houseTypeService.deleteByExample(houseTypeExample);
        String houseTypeStr = model.getHouseTypeStr();
        List<HouseType> houseTypes = JSONObject.parseArray(houseTypeStr, HouseType.class);
        for(HouseType houseType : houseTypes){
            houseType.setId(IdUtil.simpleUUID());
            houseType.setDiscId(model.getId());
            houseTypeService.insertSelective(houseType);
        }
        int result = updateByPrimaryKeySelective(model);
        return result;
    }

    public ParkFloorDiscRespson getById(String id) {
        ParkFloorDisc parkFloorDisc = selectByPrimaryKey(id);
        ParkFloorDiscRespson parkFloorDiscRespson = new ParkFloorDiscRespson();
        BeanUtil.copyProperties(parkFloorDisc,parkFloorDiscRespson);
        HouseTypeExample houseTypeExample = new HouseTypeExample();
        houseTypeExample.createCriteria().andDiscIdEqualTo(id);
        List<HouseType> houseTypes = houseTypeService.selectByExample(houseTypeExample);
        if(CollectionUtils.isNotEmpty(houseTypes)){
            String json = JSON.toJSON(houseTypes).toString();
            parkFloorDiscRespson.setHouseTypeStr(json);
        }
        return parkFloorDiscRespson;
    }

    public Map<String,String> getNameIdMap() {
        Map<String, String> map = new HashMap<>();
        List<ParkFloorDisc> list = selectByExample(new ParkFloorDiscExample());
        if (CollectionUtils.isNotEmpty(list)) {
            map = list.stream().collect(Collectors.toMap(ParkFloorDisc::getName, ParkFloorDisc::getId, (v1, v2) ->v1));
        }
        return map;
    }

    public void insertBatch(List<ParkFloorDisc> list){

    }


}
