package com.idea.config;

import com.github.pagehelper.PageInterceptor;
import com.github.pagehelper.autoconfigure.PageHelperProperties;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import java.util.List;

/**
 * 自定义mybatis拦截器配置类，自定义拦截器如有需先于PageHelper拦截器执行，需放在这
 * 注意自定义拦截器有query，且涉及到改变入参参数值的，需在PageHelper拦截器之前执行
 * 本配置只适用于pagehelper-spring-boot-autoconfigure 1.2.x版本，1.3以上的不知道咋写，谁想用谁来写^_^
 * @classname CustomMybatisInterceptorConfig
 * @date 2023年10月13日
 */
@Configuration
public class MybatisInterceptorConfig {
    @Autowired
    private List<SqlSessionFactory> sqlSessionFactoryList;
    @Autowired
    private PageHelperProperties properties;


    @PostConstruct
    public void addPageInterceptor() {
        SqlInterceptor baseEntityInterceptor = new SqlInterceptor();//自定义拦截器
        PageInterceptor pageInterceptor = new PageInterceptor();
        pageInterceptor.setProperties(this.properties.getProperties());
        //在把PageHelper特殊配置放进去，由于close-conn 利用上面方式时，属性名就是 close-conn 而不是 closeConn，所以需要额外的一步
        for (SqlSessionFactory sqlSessionFactory : sqlSessionFactoryList) {
            sqlSessionFactory.getConfiguration().addInterceptor(baseEntityInterceptor);//后注册自定义拦截器，后注册的先执行（用到了责任链模式）
        }
    }

}
